// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TicTacToeGame.hpp"
#include "game/DawnGame.hpp"
#include "scene/components/example/ExampleSpin.hpp"
#include "scene/components/physics/3d/CubeCollider.hpp"

using namespace Dawn;

TicTacToeGame::TicTacToeGame(SceneItem *item) : SceneItemComponent(item) {}

void TicTacToeGame::onStart() {
  this->tiles = getScene()->findComponents<TicTacToeTile>();

  getScene()->eventSceneUpdate.addListener(this, &TicTacToeGame::onSceneUpdate);
}

void TicTacToeGame::onSceneUpdate() {
  // Get mouse in screen space.
  auto mouse = getGame()->inputManager.getAxis2D(INPUT_BIND_MOUSE_X, INPUT_BIND_MOUSE_Y);
  mouse *= 2.0f;
  mouse -= glm::vec2(1, 1);

  Camera *camera = getScene()->findComponent<Camera>();
  if(camera == nullptr) return;

  struct Ray3D ray;
  ray.origin = camera->transform->getWorldPosition();
  ray.direction = camera->getRayDirectionFromScreenSpace(mouse);
  std::cout << "World " << ray.origin.x << ", " << ray.origin.y << ", "  << ray.origin.z << std::endl;

  auto results = getPhysics()->raycast3DAll(ray);
  auto itResult = results.begin();
  while(itResult != results.end()) {
    auto result = *itResult;
    getScene()->debugRay((struct SceneDebugRay){
      .start = result.point,
      .direction = result.normal,
      .color = COLOR_BLUE
    });
    ++itResult;
  }
}